package com.zehong.gassdevicereport.netty;

import com.zehong.gassdevicereport.constant.Constant;
import com.zehong.gassdevicereport.entity.ReciveReportData;
import com.zehong.gassdevicereport.entity.TDeviceReportData;
import com.zehong.gassdevicereport.netty.decryptreported.flowmeter.DecryptReportedData;
import com.zehong.gassdevicereport.netty.decryptreported.flowmeter.FlowmeterDecryReportedDataStrategy;
import com.zehong.gassdevicereport.netty.decryptreported.PressureDecryptReportedData;
import com.zehong.gassdevicereport.service.ITDeviceReportDataService;
import com.zehong.gassdevicereport.utils.CrcUtil;
import com.zehong.gassdevicereport.utils.HexUtils;
import com.zehong.gassdevicereport.utils.RedisUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.socket.SocketChannel;
import io.netty.util.CharsetUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.UnsupportedEncodingException;
import java.util.Date;

/**
 * 物联网 开启检测端口 并写入数据库
 */
@Component
@Sharable
public class ServerHandler extends ChannelInboundHandlerAdapter {

    private Logger logger = LoggerFactory.getLogger(ServerHandler.class);

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private ITDeviceReportDataService itDeviceReportDataService;

    private final static String METER = "4";

    private final static String METER_ALARM = "1";

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        SocketChannel channel = (SocketChannel) ctx.channel();
        logger.info("链接报告开始");
        logger.info("链接报告信息：有一客户端链接到本服务端");
        logger.info("链接报告IP:{}", channel.localAddress().getHostString());
        logger.info("链接报告Port:{}", channel.localAddress().getPort());
        logger.info("链接报告完毕");
        //通知客户端链接建立成功
        String str = "通知客户端链接建立成功" + " " + new Date() + " " + channel.localAddress().getHostString() + "\r\n";
        ctx.writeAndFlush(str);
    }

    /**
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        logger.info("客户端断开链接{}", ctx.channel().localAddress().toString());
    }
    /**
     * 获取数据
     * @param ctx 上下文
     * @param msg 获取的数据
     * @throws UnsupportedEncodingException
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        //msg为接收到的客户端传递的数据   个人这边直接传的json 数据
        ByteBuf readMessage= (ByteBuf) msg;
        String msgStr = readMessage.toString(CharsetUtil.UTF_8);
        logger.info("receiveMsgstr======="+msgStr);
        if(msgStr.length() != 124){
            return;
        }
        byte[] bytes = HexUtils.hexToByteArray( msgStr.substring(0,msgStr.length()-4));
        String checkVule = CrcUtil.getCrcToHex(bytes,true);
        String checkCode = msgStr.substring(msgStr.length()-4,msgStr.length());
        if(!checkVule.equals(checkCode)){
            return;
        }

        SocketChannel channel=(SocketChannel)ctx.channel();
        int port = channel.localAddress().getPort();
        TDeviceReportData tDeviceReportData = new TDeviceReportData();
        if(2396 == port){
            DecryptReportedData  decryptReportedData = FlowmeterDecryReportedDataStrategy.getDecryptReportedData(msgStr);
            if(null == decryptReportedData){
                logger.error("未获取到流量计类型");
                throw new Exception("未获取到流量计类型");
            }
            tDeviceReportData = decryptReportedData.getReportedData();
        }
        if (65011 == port){
            tDeviceReportData = new PressureDecryptReportedData(msgStr).getReportedData();
        }
        //缓存设备信息
        ReciveReportData reciveReportData = new ReciveReportData();
        reciveReportData.setDeviceCode(tDeviceReportData.getDeviceNum());
        reciveReportData.setDeviceType(METER);
        reciveReportData.setAlarmType(METER_ALARM);
        reciveReportData.setPressure(tDeviceReportData.getPressure());
        redisUtil.set(Constant.DEVICE_REDIS_KEY + reciveReportData.getDeviceCode(),tDeviceReportData);
        //设备上报数据入库
        itDeviceReportDataService.insertTDeviceReportData(tDeviceReportData);

       /* String rmsg="0";//返回失败的信息
        ByteBuf message= Unpooled.copiedBuffer(rmsg.getBytes());//处理返回的信息
        ctx.writeAndFlush(message);//返回信息
        //刷新缓存区
        ctx.flush();*/
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }
}
